var keyCodeMap = {
    8:"backspace", 9:"tab", 13:"return", 16:"shift", 17:"ctrl", 18:"alt", 19:"pausebreak", 20:"capslock", 27:"escape", 32:"space", 33:"pageup",
    34:"pagedown", 35:"end", 36:"home", 37:"left", 38:"up", 39:"right", 40:"down", 43:"+", 44:"printscreen", 45:"insert", 46:"delete",
    48:"0", 49:"1", 50:"2", 51:"3", 52:"4", 53:"5", 54:"6", 55:"7", 56:"8", 57:"9", 59:";",
    61:"=", 65:"a", 66:"b", 67:"c", 68:"d", 69:"e", 70:"f", 71:"g", 72:"h", 73:"i", 74:"j", 75:"k", 76:"l",
    77:"m", 78:"n", 79:"o", 80:"p", 81:"q", 82:"r", 83:"s", 84:"t", 85:"u", 86:"v", 87:"w", 88:"x", 89:"y", 90:"z",
    96:"0", 97:"1", 98:"2", 99:"3", 100:"4", 101:"5", 102:"6", 103:"7", 104:"8", 105:"9",
    106: "*", 107:"+", 109:"-", 110:".", 111: "/",
    112:"f1", 113:"f2", 114:"f3", 115:"f4", 116:"f5", 117:"f6", 118:"f7", 119:"f8", 120:"f9", 121:"f10", 122:"f11", 123:"f12",
    144:"numlock", 145:"scrolllock", 186:";", 187:"=", 188:",", 189:"-", 190:".", 191:"/", 192:"`", 219:"[", 220:"\\", 221:"]", 222:"'"
}
var shortcut_set = false;
var selected_key = undefined;


function callback(name, data) {
    if (!data) data = '';
    window.location.href = 'skp:' + name + '@' + data;
}


function init() {
    var w = $( window ).width();
    var h = $( window ).height();
    var data = '['+w+','+h+']';
    callback('init', data);
}


function get_open_sections() {
    var data = '';
    $( '.collapsible' ).each( function(index) {
        if ($( this ).hasClass('collapse-open')) {
            data += ',' + this.id;
        }
    });
    return data.substring(1);
}


function set_open_sections(opts) {
    var sections = opts.split(',');
    $( '.collapsible' ).each( function() {
        if (jQuery.inArray( this.id, sections ) == -1) {
            $( this ).collapsible('loadClose');
        } else {
            $( this ).collapsible('loadOpen');
        }
    });
}


function adjust_size() {
    var w = 256 //$( '.collapsible' ).width() + 20;
    var h = $( 'body' ).height();
    var data = '['+w+','+h+']';
    callback('adjust_size', data);
}


function drag_start() {
    callback('drag_start', null);
}


function drag(dx,dy) {
    var data = '['+dx+','+dy+']';
    callback('drag', data);
}


function drag_end() {
    callback('drag_end', null);
}


function collapsible_changed() {
    var data = get_open_sections();
    callback('collapsible_changed', data);
}


function input_changed(id, val) {
    var data = '['+':'+ id+','+val+']';
    callback('input_changed', data);
}


function caption_changed(element) {
   var str = element.value.replace(/"|'|\\|\n|\r|^\s+|\s+$/g, '');
   element.value = str;
   callback('caption_changed', str);
}


function button_clicked(id) {
    callback('button_clicked', id);
}

function select_changed() {
    var val = document.getElementById('sel1').value;
    callback('select_changed', val);
}

function shortcut_changed() {
    var key = document.getElementById('sel1').value;
    var val = document.getElementById('txt2').value;
    if (!key) return;
    if (val != '' && !shortcut_set) return;
    var data = '[:'+key+',"'+val+'"]';
    callback('shortcut_changed', data);
}


$(document).ready( function() {
    // Initialize
    window.setTimeout(init, 0);

    // Collapsible management
    $.fn.slideFadeToggle = function(speed, easing, callback) {
        return this.animate({opacity: 'toggle', height: 'toggle'}, speed, easing, callback);
    };
    $( '.collapsible' ).collapsible({
        defaultOpen: '',
        speed: 'normal',
        bind: 'click',
        animateOpen: function (elem, opts) {
            elem.next().slideFadeToggle(opts.speed);
            collapsible_changed();
        },
        animateClose: function (elem, opts) {
            elem.next().slideFadeToggle(opts.speed);
            collapsible_changed();
        },
        loadOpen: function (elem) {
            elem.next().show();
        },
        loadClose: function (elem, opts) {
            elem.next().hide();
        }
    });

    // Adjust size
    jQuery.resize.delay = 10;
    $( 'body' ).resize( function() {
        setTimeout(adjust_size, 0);
    });

    // Detect Drag
    var md = false;
    var pos = [0,0];
    $( '.collapsible' ).mousedown(function(e) {
        md = true;
        var x = e.clientX;
        var y = e.clientY;
        pos = [x,y];
        drag_start();
    });
    $(document).mousemove(function(e) {
        if (!md) return;
        var x = e.clientX;
        var y = e.clientY;
        var dx = x-pos[0];
        var dy = y-pos[1];
        if ((dx == 0) && (dy == 0)) return;
        drag(dx,dy);
    });
    $(document).mouseup(function(e) {
        if (md) {
            md = false;
            drag_end();
        }
    });

    // Process checkbox and radio input triggers.
    $( 'input[type=checkbox], input[type=radio]' ).change( function() {
        input_changed(this.id, this.checked);
    });

    // Process text 1 input
    $( '#txt1' ).keydown( function(e) {
        e = e || window.event;
        var k = e.charCode || e.keyCode || 0;
        if (k == 13) caption_changed(this);
        return true;
    });
    $( '#txt1' ).change( function() {
        caption_changed(this);
    });

    // Process text 2 input
    $( '#txt2' ).keydown( function(e) {
        e = e || window.event;
        var k = e.charCode || e.keyCode || 0;
        var changed = false;
        if ((k == 8) || (k >= 16 && k <= 18)) {
            shortcut_set = false;
            changed = true;
        } else if ((k == 32) || (k >= 48 && k <= 57) || (k >= 65 && k <= 90) || (k >= 112 && k <= 123)) {
            selected_key = keyCodeMap[k];
            shortcut_set = true;
            changed = true;
        }
        if (changed) {
            this.value = (
                (e.ctrlKey ? 'ctrl+' : '') +
                (e.shiftKey ? 'shift+' : '') +
                (e.altKey ? 'alt+' : '') +
                (selected_key ? selected_key : '')
            ).toUpperCase();
        }
        if (k == 13) {
            shortcut_changed();
        }
        return false;
    });
    $( '#txt2' ).keyup( function(e) {
        e = e || window.event;
        var k = e.charCode || e.keyCode || 0;
        if (keyCodeMap[k] == selected_key) selected_key = undefined;
        if ((k >= 16 && k <= 18) && !shortcut_set) {
            this.value = '';
        }
    });

    // Process button clicks
    $( 'button' ).click( function() {
        if (this.name == 'submit') {
            shortcut_changed();
        } else {
            button_clicked(this.name);
        }
    });

    // Process select
    $( '#sel1' ).click(function() {
        select_changed();
    });
    $( '#sel1' ).change(function() {
        select_changed();
    });
});
